// desc.cpp:
//
// security descriptor implementation
//
// jim frost 05.02.95

#include <stdio.h>
#include "security.hpp"

SecurityDescriptor*
GetFileSecurityDescriptor(const char* filename)
{
  DWORD attributes = (OWNER_SECURITY_INFORMATION |
		      GROUP_SECURITY_INFORMATION |
		      SACL_SECURITY_INFORMATION |
		      DACL_SECURITY_INFORMATION);
  char buffer[1024];
  int err;
  DWORD size = sizeof(buffer);
  char* bp = buffer;
  SecurityIdentifier* owner = NULL;
  SecurityIdentifier* group = NULL;
  AccessControlList* system_acl = NULL;
  AccessControlList* discretionary_acl = NULL;

  while (!GetFileSecurity(filename, attributes, bp, size, &size)) {
	  err = GetLastError();
    switch (err) {
    case ERROR_PRIVILEGE_NOT_HELD:
      // it's common for a user not to have permission to see the
      // system ACL list.  if that happens we try again without
      // the SACL attribute.  if it happens even without the SACL
      // attribute then we're out of luck....

      if (attributes & SACL_SECURITY_INFORMATION)
	attributes &= ~SACL_SECURITY_INFORMATION;
      else
	return NULL;
      break;
    case ERROR_INSUFFICIENT_BUFFER:
      if (bp != buffer)
	delete[] bp;
      bp = new char[size];
      break;
    default:
      return NULL;
    }
  }

  PSID sid;
  BOOL defaulted;

  if (!GetSecurityDescriptorOwner(bp, &sid, &defaulted)) {
    if (bp != buffer)
      delete[] bp;
    return NULL;
  }
  owner = new SecurityIdentifier(sid);

  if (GetSecurityDescriptorGroup(bp, &sid, &defaulted))
    group = new SecurityIdentifier(sid);

  BOOL has_acl;
  ACL* acl;

  if (attributes & SACL_SECURITY_INFORMATION) {
    GetSecurityDescriptorSacl(bp, &has_acl, &acl, &defaulted);
    if (has_acl)
      system_acl = new AccessControlList(acl);
  }

  GetSecurityDescriptorDacl(bp, &has_acl, &acl, &defaulted);
  if (has_acl)
    discretionary_acl = new AccessControlList(acl);

  return new SecurityDescriptor(owner, group, system_acl, discretionary_acl);
}

BOOL
SetFileSecurityDescriptor(const char* filename, DWORD attributes, SecurityDescriptor* sd)
{
  char desc_buffer[1024]; // dunno how to tell if this is too small

  InitializeSecurityDescriptor(desc_buffer, SECURITY_DESCRIPTOR_REVISION);
  SetSecurityDescriptorOwner(desc_buffer, sd->GetOwner()->sid, FALSE);
  if (sd->GetGroup() != NULL)
    SetSecurityDescriptorGroup(desc_buffer, sd->GetGroup()->sid, FALSE);
  if (sd->GetSystemAcl() == NULL)
    SetSecurityDescriptorSacl(desc_buffer, FALSE, NULL, FALSE);
  else
    SetSecurityDescriptorSacl(desc_buffer, TRUE, sd->GetSystemAcl()->acl, FALSE);
  if (sd->GetDiscretionaryAcl() == NULL)
    SetSecurityDescriptorDacl(desc_buffer, FALSE, NULL, FALSE);
  else
    SetSecurityDescriptorDacl(desc_buffer, TRUE, sd->GetDiscretionaryAcl()->acl, FALSE);

  return SetFileSecurity(filename, attributes, desc_buffer);
}

SecurityDescriptor::SecurityDescriptor(void)
{
  owner = NULL;
  group = NULL;
  system_acl = NULL;
  discretionary_acl = NULL;
}

SecurityDescriptor::SecurityDescriptor(SecurityIdentifier* _owner,
				       SecurityIdentifier* _group,
				       AccessControlList* _system_acl,
				       AccessControlList* _discretionary_acl)
{
  owner = _owner;
  group = _group;
  system_acl = _system_acl;
  discretionary_acl = _discretionary_acl;
}

SecurityDescriptor::SecurityDescriptor(const SecurityDescriptor* _sd)
{
  SetOwner(_sd->owner);
  SetGroup(_sd->group);
  SetSystemAcl(_sd->system_acl);
  SetDiscretionaryAcl(_sd->discretionary_acl);
}

SecurityDescriptor::~SecurityDescriptor(void)
{
  if (owner != NULL)
    delete owner;
  if (group != NULL)
    delete group;
  if (system_acl != NULL)
    delete system_acl;
  if (discretionary_acl != NULL)
    delete discretionary_acl;
}

void
SecurityDescriptor::SetOwner(SecurityIdentifier* new_owner)
{
  if (owner != NULL)
    delete owner;
  if (new_owner == NULL)
    owner = NULL;
  else
    owner = new SecurityIdentifier(new_owner);
}

void
SecurityDescriptor::SetGroup(SecurityIdentifier* new_group)
{
  if (group != NULL)
    delete group;
  if (new_group == NULL)
    group = NULL;
  else
    group = new SecurityIdentifier(new_group);
}

void
SecurityDescriptor::SetSystemAcl(AccessControlList* new_system_acl)
{
  if (system_acl != NULL)
    delete system_acl;
  if (new_system_acl == NULL)
    system_acl = NULL;
  else
    system_acl = new AccessControlList(new_system_acl);
}

void
SecurityDescriptor::SetDiscretionaryAcl(AccessControlList* new_discretionary_acl)
{
  if (discretionary_acl != NULL)
    delete discretionary_acl;
  if (new_discretionary_acl == NULL)
    discretionary_acl = NULL;
  else
    discretionary_acl = new AccessControlList(new_discretionary_acl);
}
