// security.hpp:
//
// a variety of classes used to manage security objects
//
// jim frost 04.06.95

#include <windows.h>

class SecurityIdentifier;
class AccessControlEntry;
class AccessControlList;
class SecurityDescriptor;

SecurityIdentifier* CreateSecurityIdentifier(const char*);

class SecurityIdentifier {
  friend AccessControlList;
  friend SecurityIdentifier* CreateSecurityIdentifier(const char*);
  friend SecurityDescriptor* GetFileSecurityDescriptor(const char*);
  friend BOOL SetFileSecurityDescriptor(const char*, DWORD, SecurityDescriptor*);
private:
  PSID sid;
  char* account;
  char* domain;

  SecurityIdentifier(PSID);

public:
  SecurityIdentifier(const SecurityIdentifier*);
  ~SecurityIdentifier(void);

  const char* GetAccount(void);
  const char* GetDomain(void);
};

class AccessControlEntry {
private:
  SecurityIdentifier* sid;
  unsigned int type;
  unsigned int flags;
  unsigned int mask;

  AccessControlEntry(const void*);

public:
  AccessControlEntry(void);
  ~AccessControlEntry(void);

  unsigned int GetType(void);
  void         SetType(unsigned int);

  unsigned int GetFlags(void);
  void         SetFlags(unsigned int);
  void         AddFlags(unsigned int);
  void         RemoveFlags(unsigned int);

  unsigned int GetMask(void);
  void         SetMask(unsigned int);
  void         AddMask(unsigned int);
  void         RemoveMask(unsigned int);

  SecurityIdentifier* GetSecurityIdentifier(void);
  void                SetSecurityIdentifier(SecurityIdentifier*);
  const char*         GetAccount(void);
};

class SecurityDescriptor;

union AceUnion {
  ACE_HEADER header;
  ACCESS_ALLOWED_ACE allow;
  ACCESS_DENIED_ACE deny;
};


class AccessControlList {
  friend SecurityDescriptor;
  friend SecurityDescriptor* GetFileSecurityDescriptor(const char*);
  friend BOOL SetFileSecurityDescriptor(const char*, DWORD, SecurityDescriptor*);
private:
  ACL* acl;

  AccessControlList(const ACL*);

public:
  AccessControlList(void);
  AccessControlList(const AccessControlList*);
  ~AccessControlList(void);

  unsigned int GetLength(void);
  void AddAce(AceUnion*);
  void AppendAce(AceUnion*);
  void AddAllowAce(unsigned char, SecurityIdentifier*, DWORD);
  void AddDenyAce(unsigned char, SecurityIdentifier*, DWORD);
  void AppendAllowAce(unsigned char, SecurityIdentifier*, DWORD);
  void AppendDenyAce(unsigned char, SecurityIdentifier*, DWORD);
  void DeleteAce(unsigned int);
  unsigned int GetAce(unsigned int, AccessControlEntry&);
};

class SecurityDescriptor {
  friend SecurityDescriptor* GetFileSecurityDescriptor(const char*);
  friend BOOL SetFileSecurityDescriptor(const char*, DWORD, SecurityDescriptor*);
private:
  char* security_descriptor;
  SecurityIdentifier* owner;
  SecurityIdentifier* group;
  AccessControlList* system_acl;
  AccessControlList* discretionary_acl;

  SecurityDescriptor(SecurityIdentifier*,
		     SecurityIdentifier*,
		     AccessControlList*,
		     AccessControlList*);

public:
  SecurityDescriptor(void);
  SecurityDescriptor(const SecurityDescriptor*);
  ~SecurityDescriptor(void);

  SecurityIdentifier* GetOwner(void) { return owner; }
  void SetOwner(SecurityIdentifier*);

  SecurityIdentifier* GetGroup(void) { return group; }
  void SetGroup(SecurityIdentifier*);

  AccessControlList* GetSystemAcl(void) { return system_acl; }
  void SetSystemAcl(AccessControlList*);

  AccessControlList* GetDiscretionaryAcl(void) { return discretionary_acl; }
  void SetDiscretionaryAcl(AccessControlList*);
};



void whoami();

void restrict(LPCSTR fileName);

void protect(LPCSTR fileName);


void access(LPTSTR fileName);



